package com.example.civilizedcity.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.ResponseBean;
import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.entity.TaIssue;
import com.example.civilizedcity.entity.TaOrgIssue;
import com.example.civilizedcity.event.MessagEvent;
import com.example.civilizedcity.mapper.TaIssueMapper;
import com.example.civilizedcity.mapper.TaOrgIssueMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;
import com.example.civilizedcity.entity.TaIssueApply;
import com.example.civilizedcity.mapper.TaIssueApplyMapper;
import com.example.civilizedcity.service.TaIssueApplyService;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * 问题处理申请;(ta_issue_apply)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2022-12-27
 */
@Service
public class TaIssueApplyServiceImpl extends BaseServiceImpl<TaIssueApplyMapper, TaIssueApply> implements TaIssueApplyService {

    @Autowired
    TaIssueMapper taIssueMapper;

    @Autowired
    TaOrgIssueMapper taOrgIssueMapper;

    @Autowired
    ApplicationEventPublisher applicationEventPublisher;

    @Override
    public void createNewApply(TaIssueApply taIssueApply, SysUser sysUser) throws Exception {

        if (null == taIssueApply.getIssueId()) {
            throw new Exception("校验问题单出错");
        }

        if (!Constants.ROOT_ORG.equals(sysUser.getOrgId())) {
            // 如果是责任单位打的申请
            TaOrgIssue taOrgIssue = taOrgIssueMapper.getByIssueAndOrg(taIssueApply.getIssueId(), sysUser.getOrgId());
            if (null == taOrgIssue) {
                throw new Exception("未找到交办单位问题单");
            }

            taOrgIssueMapper.updateById(taOrgIssue);
        }

        taIssueApply.setApplyId(null);
        taIssueApply.setCreateUser(sysUser.getUserId());
        taIssueApply.setCreateUserName(sysUser.getName());
        taIssueApply.setOrgId(sysUser.getOrgId());
        taIssueApply.setCreateDate(LocalDateTime.now());

        save(taIssueApply);

        taIssueMapper.updateApply(taIssueApply.getIssueId(), taIssueApply.getApplyType(), taIssueApply.getApplyId());


//        if (Constants.APPLY_VERIFY.equals(taIssueApply.getApplyType())) {
//            taIssueMapper.updateValidateStatus(taIssueApply.getIssueId(), Constants.VERIFY_SUBMIT);
//        }

        applicationEventPublisher.publishEvent(new MessagEvent(this, taIssueApply));
    }

    @Override
    public IPage<TaIssueApply> getPageBy(IPage<TaIssueApply> pg, Integer issueId, String applyType, String orgId, String sourceType) {
        return baseMapper.getPageBy(pg, issueId, applyType, orgId, sourceType);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void verify(TaIssueApply taIssueApply, SysUser sysUser, TaIssueApply origin) throws Exception {
        // 先更新审核信息
        origin.setVerifyStatus(taIssueApply.getVerifyStatus());
        origin.setVerifyDesc(taIssueApply.getVerifyDesc());
        origin.setVerifyOrg(sysUser.getOrgId());
        origin.setVerifyUser(sysUser.getUserId());
        origin.setVerifyUserName(sysUser.getName());
        origin.setVerifyDate(LocalDateTime.now());
        updateById(origin);

        // 再判断审核类型
        switch (origin.getApplyType()) {
            case Constants.APPLY_VERIFY:
                // 问题单审核
                verifyIssue();
                break;
            case Constants.APPLY_REJECT:
                // 问题单驳回
                verifyReject();
                break;
            case Constants.APPLY_DELAY:
                // 延期申请
                verifyDelay(taIssueApply, origin);
                break;
            case Constants.APPLY_END:
                // 销单申请
                break;
            default:
                // 其他
                break;
        }

    }

    // 问题单审核
    public void verifyIssue() throws Exception {
        // TODO;
    }

    // 驳回审核
    public void verifyReject() throws Exception {
        // TODO;
    }

    // 延期审核
    public void verifyDelay(TaIssueApply taIssueApply, TaIssueApply origin) throws Exception {
        if (Constants.APPLY_STATUS_REJECT.equals(taIssueApply.getVerifyStatus())) {
            // 如果审核不通过
            // TODO;
        } else {
            // 问题单截止日期更新为申请的截止日期
            // 问题单的申请关联清空
            UpdateWrapper<TaIssue> updateWrapper = new UpdateWrapper<>();
            updateWrapper.set("expire_date", taIssueApply.getContent());
            updateWrapper.set("apply_type", null);
            updateWrapper.set("apply_id", null);
            updateWrapper.eq("issue_id", origin.getIssueId());

        }
    }

    // 问题单审核
    public void verifyIssue() throws Exception {
        // TODO;
    }


}